searchData={"items":[{"type":"task","title":"mix pillar.gen.migration","doc":"Migration Generator (very simple)\n\n```\n$bash> mix pillar.gen.migration new_table\n=> Migration generated priv/pillar_migrations/1592084569_new_table.exs\n```","ref":"Mix.Tasks.Pillar.Gen.Migration.html"},{"type":"module","title":"Pillar.BulkInsertBuffer","doc":"This module provides functionality for bulk inserts and buffering records\n\n```elixir\ndefmodule BulkToLogs do\n  use Pillar.BulkInsertBuffer,\n    pool: ClickhouseMaster,\n    table_name: \"logs\",\n    interval_between_inserts_in_seconds: 5,\n    on_errors: &__MODULE__.dump_to_file/2\n\n  def dump_to_file(_result, records) do\n    File.write(\"bad_inserts/2025-05-27 10:31:56.461489Z\", inspect(records))\n  end\nend\n```\n\n```elixir\n:ok = BulkToLogs.insert(%{value: \"online\", count: 133, datetime: DateTime.utc_now()})\n:ok = BulkToLogs.insert(%{value: \"online\", count: 134, datetime: DateTime.utc_now()})\n:ok = BulkToLogs.insert(%{value: \"online\", count: 132, datetime: DateTime.utc_now()})\n....\n\n# all this records will be inserted with 5 second interval\n```","ref":"Pillar.BulkInsertBuffer.html"},{"type":"module","title":"Pillar.Connection","doc":"Structure with connection config, such as host, port, user, password and other","ref":"Pillar.Connection.html"},{"type":"function","title":"Pillar.Connection.new/1","doc":"Generates Connection from typical connection string:\n\n```\n%Pillar.Connection{} = Pillar.Connection.new(\"https://user:password@localhost:8123/some_database\")\n\n# in this case \"default\" database is used\n%Pillar.Connection{} = Pillar.Connection.new(\"https://localhost:8123\")\n```","ref":"Pillar.Connection.html#new/1"},{"type":"type","title":"Pillar.Connection.t/0","doc":"","ref":"Pillar.Connection.html#t:t/0"},{"type":"function","title":"Pillar.Connection.url_from_connection/2","doc":"","ref":"Pillar.Connection.html#url_from_connection/2"},{"type":"module","title":"Pillar.HttpClient.Response","doc":"Wrapper for HTTP response","ref":"Pillar.HttpClient.Response.html"},{"type":"type","title":"Pillar.HttpClient.Response.t/0","doc":"","ref":"Pillar.HttpClient.Response.html#t:t/0"},{"type":"module","title":"Pillar.HttpClient.TransportError","doc":"Wrapper for HTTP transport errors, such as timeout and etc","ref":"Pillar.HttpClient.TransportError.html"},{"type":"type","title":"Pillar.HttpClient.TransportError.t/0","doc":"","ref":"Pillar.HttpClient.TransportError.html#t:t/0"},{"type":"module","title":"Pillar.Migrations.Generator","doc":"Migration generator, used at Mix.Tasks","ref":"Pillar.Migrations.Generator.html"},{"type":"function","title":"Pillar.Migrations.Generator.migration_filepath/2","doc":"","ref":"Pillar.Migrations.Generator.html#migration_filepath/2"},{"type":"function","title":"Pillar.Migrations.Generator.migration_template/1","doc":"","ref":"Pillar.Migrations.Generator.html#migration_template/1"},{"type":"module","title":"Pillar.MigrationsMacro","doc":"Migration's mechanism\n\nFor generation migration files use task `Mix.Tasks.Pillar.Gen.Migration`\n\nFor launching migration define own mix task or release task with code below:\n\n```\nconn = Pillar.Connection.new(connection_string)\nPillar.Migrations.migrate(conn)\n```","ref":"Pillar.MigrationsMacro.html"},{"type":"module","title":"Pillar.Util","doc":"","ref":"Pillar.Util.html"},{"type":"function","title":"Pillar.Util.has_input_format_json_read_numbers_as_strings?/1","doc":"","ref":"Pillar.Util.html#has_input_format_json_read_numbers_as_strings?/1"},{"type":"function","title":"Pillar.Util.needs_decimal_zero_for_integers_in_json?/1","doc":"","ref":"Pillar.Util.html#needs_decimal_zero_for_integers_in_json?/1"},{"type":"extras","title":"Pillar","doc":"# Pillar\n\n[![github.com](https://github.com/am-kantox/pillar/workflows/build/badge.svg?branch=master)](https://github.com/am-kantox/pillar/actions)\n[![hex.pm](https://img.shields.io/badge/docs-hexpm-blue.svg)](https://hexdocs.pm/pillar)\n[![hex.pm](https://img.shields.io/hexpm/v/pillar.svg)](https://hex.pm/packages/pillar)\n[![hex.pm](https://img.shields.io/hexpm/dt/pillar.svg)](https://hex.pm/packages/pillar)\n[![hex.pm](https://img.shields.io/hexpm/l/pillar.svg)](https://hex.pm/packages/pillar)\n[![github.com](https://img.shields.io/github/last-commit/am-kantox/pillar.svg)](https://github.com/am-kantox/pillar/commits/master)\n\nElixir client for [ClickHouse](https://clickhouse.tech/), a fast open-source\nOnline Analytical Processing (OLAP) database management system.\n\n# Features\n\n  - [Direct Usage with connection structure](#direct-usage-with-connection-structure)\n  - [Pool of workers](#pool-of-workers)\n  - [Async insert](#async-insert)\n  - [Buffer for periodical bulk inserts](#buffer-for-periodical-bulk-inserts)\n  - [Migrations](#migrations)\n  - [DateTime Timezones](#timezones)\n  - [Switching between HTTP adapters](#http-adapters)","ref":"readme.html"},{"type":"extras","title":"Usage - Pillar","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Direct Usage with connection structure - Pillar","doc":"```elixir\nconn = Pillar.Connection.new(\"http://user:password@localhost:8123/database\")\n\n# Params are passed in brackets {} in SQL query, and map strtucture does fill\n# query by values.\nsql = \"SELECT count(*) FROM users WHERE lastname = {lastname}\"\n\nparams = %{lastname: \"Smith\"}\n\n{:ok, result} = Pillar.query(conn, sql, params)\n\nresult\n#=> [%{\"count(*)\" => 347}]\n\n```","ref":"readme.html#direct-usage-with-connection-structure"},{"type":"extras","title":"Pool of workers - Pillar","doc":"Recommended usage, because of limited connections and supervised workers.\n\n```elixir\ndefmodule ClickhouseMaster do\n  use Pillar,\n    connection_strings: [\n      \"http://user:password@host-master-1:8123/database\",\n      \"http://user:password@host-master-2:8123/database\"\n    ],\n    name: __MODULE__,\n    pool_size: 15\nend\n\nClickhouseMaster.start_link()\n\n{:ok, result} = ClickhouseMaster.select(sql, %{param: value})\n```","ref":"readme.html#pool-of-workers"},{"type":"extras","title":"Async insert - Pillar","doc":"```elixir\nconnection = Pillar.Connection.new(\"http://user:password@host-master-1:8123/database\")\n\nPillar.async_insert(connection, \"INSERT INTO events (user_id, event) SELECT {user_id}, {event}\", %{\n  user_id: user.id,\n  event: \"password_changed\"\n}) # => :ok\n```","ref":"readme.html#async-insert"},{"type":"extras","title":"Buffer for periodical bulk inserts - Pillar","doc":"For this feature required [Pool of workers](#pool-of-workers).\n\n```elixir\ndefmodule BulkToLogs do\n  use Pillar.BulkInsertBuffer,\n    pool: ClickhouseMaster,\n    table_name: \"logs\",\n    # interval_between_inserts_in_seconds, by default -> 5\n    interval_between_inserts_in_seconds: 5,\n    # on_errors is optional\n    on_errors: &__MODULE__.dump_to_file/2\n\n  @doc \"\"\"\n  dump to file function store failed inserts into file \n  \"\"\"\n  def dump_to_file(_result, records) do\n    File.write(\"bad_inserts/#{DateTime.utc_now()}\", inspect(records))\n  end\n\n  @doc \"\"\"\n  retry insert is dangerous (but it is possible and listed as proof of concept)\n\n  this function may be used in `on_errors` option\n  \"\"\"\n  def retry_insert(_result, records) do\n    __MODULE__.insert(records)\n  end\nend\n```\n\n```elixir\n:ok = BulkToLogs.insert(%{value: \"online\", count: 133, datetime: DateTime.utc_now()})\n:ok = BulkToLogs.insert(%{value: \"online\", count: 134, datetime: DateTime.utc_now()})\n:ok = BulkToLogs.insert(%{value: \"online\", count: 132, datetime: DateTime.utc_now()})\n....\n\n# All this records will be inserted with 5 second interval.\n```\n\n*on_errors* parameter allows you to catch any error of bulk insert (for example: one of batch is bad or clickhouse was not available )","ref":"readme.html#buffer-for-periodical-bulk-inserts"},{"type":"extras","title":"Migrations - Pillar","doc":"Migrations can be generated with mix task `mix pillar.gen.migration migration_name`.\n\nMulti-statement migration Example for this [UseCase](https://github.com/balance-platform/pillar/issues/61)\n\n```elixir\ndefmodule Pillar.Migrations.CreateMultipleTables do\n  def up do\n    # for MultiStatement migration result of this function should be List of Strings  \n    (0..4) |> Enum.map(fn i ->\n      \"CREATE TABLE IF NOT EXISTS shard_#{i} (field FixedString(10)) ENGINE = Memory\"\n    end)\n  end\nend\n```\n\n```bash\nmix pillar.gen.migration events_table\n```\n\nBut for launching them we have to write own task, like this:\n\n```elixir\ndefmodule Mix.Tasks.MigrateClickhouse do\n  use Mix.Task\n  def run(_args) do\n    connection_string = Application.get_env(:my_project, :clickhouse_url)\n    conn = Pillar.Connection.new(connection_string)\n    Pillar.Migrations.migrate(conn)\n  end\nend\n```\n\nAnd launch this via command.\n\n```bash\nmix migrate_clickhouse\n```","ref":"readme.html#migrations"},{"type":"extras","title":"Timezones - Pillar","doc":"In order to be able to use Timezones add timezones database to your project and configure your app:\n\n```elixir\nconfig :elixir, :time_zone_database, Tzdata.TimeZoneDatabase\n```\n\nDetails here https://hexdocs.pm/elixir/1.12/DateTime.html#module-time-zone-database","ref":"readme.html#timezones"},{"type":"extras","title":"HTTP Adapters - Pillar","doc":"If you have problems with default Pillar HTTP Adapter (Mint over Tesla), you can use alternative one, based on :httpc or define your own and pass it\nthrough config.\n\n```\nconfig :pillar, Pillar.HttpClient, http_adapter: Pillar.HttpClient.TeslaMintAdapter\n```\n\nAdapter should define one function `post/3` and return 2 possible results (`%Pillar.HttpClient.Response{}`, `%Pillar.HttpClient.TransportError{}`)\n\n# Contribution\n\nFeel free to make a pull request. All contributions are appreciated!","ref":"readme.html#http-adapters"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}